@@ -612,7 +612,7 @@ void construct_cr(double *x,int *nx,double *k,int *nk,double *X,double *S,double
 } 
 
 void construct_tprs(double *x,int *d,int *n,double *knt,int *nk,int *m,int *k,double *X,double *S,
-                    double *UZ,double *Xu,int *nXu,double *C,int *max_knots)
+                    double *UZ,double *Xu,int *nXu,double *C)
 /* inputs: 
    x contains the n values of each of the d covariates, stored end to end
    knt contains the nk knot locations packed as x
@@ -637,7 +637,7 @@ void construct_tprs(double *x,int *d,int *n,double *knt,int *nk,int *m,int *k,do
   { kk=(double **)calloc((size_t)(*d),sizeof(double*));
     for (i=0;i<*d;i++) kk[i]=knt + i * *nk;
   }
-  tprs_setup(xx,kk,*m,*d,*n,*k,1,&Xm,&Sm,&UZm,&Xum,*nk,*max_knots); /* Do actual setup */
+  tprs_setup(xx,kk,*m,*d,*n,*k,1,&Xm,&Sm,&UZm,&Xum,*nk); /* Do actual setup */
   RArrayFromMatrix(X,Xm.r,&Xm);
   RArrayFromMatrix(S,Sm.r,&Sm);
   RArrayFromMatrix(UZ,UZm.r,&UZm);  
@@ -1000,23 +1000,24 @@ void MinimumSeparation(double *gx,double *gy,int *gn,double *dx,double *dy, int
 
 
 
-void RuniqueCombs(double *X,int *r, int *c)
+void RuniqueCombs(double *X,int *ind,int *r, int *c)
 
 /* X is a matrix. This routine finds its unique rows and strips out the 
    duplicates. This is useful for finding out the number of unique covariate
    combinations present in a set of data. */
 
 { matrix B,Xd;
-  int i,*ind;
+  int i,*ind1;
   B=Rmatrix(X,(long)(*r),(long)(*c));
   Xd=initmat(B.r,B.c+1);
   Xd.c--;mcopy(&B,&Xd);freemat(B);Xd.c++;
   for (i=0;i<Xd.r;i++) Xd.M[i][Xd.c-1]=(double)i;
-  ind=Xd_strip(&Xd);
+  ind1=Xd_strip(&Xd);
+  for (i=0;i<*r;i++) ind[i] = ind1[i]; /* copy index for return */
   Xd.c--; /* hide index array  */
   RArrayFromMatrix(X,Xd.r,&Xd);  /* NOTE: not sure about rows here!!!! */
   *r = (int)Xd.r; 
-  freemat(Xd);free(ind);
+  freemat(Xd);free(ind1);
 #ifdef MEM_CHECK
   dmalloc_log_unfreed();  dmalloc_verify(NULL);
 #endif 
@@ -1109,8 +1110,9 @@ else
 #endif
 }
 
-void  RPCLS(double *Xd,double *pd,double *yd, double *wd,double *Aind,double *bd,double *Afd,double *Hd,double *Sd,
-      int *off,int *dim,double *theta, int *m,int *nar)
+void  RPCLS(double *Xd,double *pd,double *yd, double *wd,double *Aind,double *bd,
+            double *Afd,double *Hd,double *Sd,
+            int *off,int *dim,double *theta, int *m,int *nar)
 
 /* Interface routine for PCLS the constrained penalized weighted least squares solver.
    nar is an array of dimensions. Let:
